@@ -2,8 +2,10 @@ package httptransport
 
 import (
 	"context"
+	"fmt"
 	"net"
 	"net/http"
+	"time"
 
 	"github.com/quay/zlog"
 	othttp "go.opentelemetry.io/contrib/instrumentation/net/http/otelhttp"
@@ -16,6 +18,7 @@ import (
 	"github.com/quay/clair/v4/indexer"
 	"github.com/quay/clair/v4/matcher"
 	intromw "github.com/quay/clair/v4/middleware/introspection"
+	"github.com/quay/clair/v4/middleware/rate"
 	notifier "github.com/quay/clair/v4/notifier/service"
 )
 
@@ -38,8 +41,7 @@ const (
 	OpenAPIV1Path           = "/openapi/v1"
 )
 
-// Server is the primary http server
-// Clair exposes it's functionality on.
+// Server is the primary http server Clair exposes its functionality on.
 type Server struct {
 	// Server embeds a http.Server and http.ServeMux.
 	// The http.Server will be configured with the ServeMux on successful
@@ -56,8 +58,8 @@ type Server struct {
 func New(ctx context.Context, conf config.Config, indexer indexer.Service, matcher matcher.Service, notifier notifier.Service) (*Server, error) {
 	serv := &http.Server{
 		Addr: conf.HTTPListenAddr,
-		// use the passed in global context as the base context
-		// for all http requests handled by this server
+		// use the passed in global context as the base context for all http
+		// requests handled by this server
 		BaseContext: func(net.Listener) context.Context { return ctx },
 	}
 	mux := http.NewServeMux()
@@ -107,7 +109,7 @@ func New(ctx context.Context, conf config.Config, indexer indexer.Service, match
 	// attach HttpTransport to server, this works because we embed http.ServeMux
 	t.Server.Handler = t
 
-	// add endpoint authentication if configured add auth. must happen after
+	// Add endpoint authentication if configured to add auth. Must happen after
 	// mux was configured for given mode.
 	if conf.Auth.Any() {
 		err := t.configureWithAuth(ctx)
@@ -121,16 +123,15 @@ func New(ctx context.Context, conf config.Config, indexer indexer.Service, match
 	return t, nil
 }
 
-// configureDiscovery() creates a discovery handler
-// for serving the v1 open api specification
+// ConfigureDiscovery creates a discovery handler for serving the v1 OpenAPI
+// specification.
 func (t *Server) configureDiscovery(_ context.Context) error {
 	t.Handle(OpenAPIV1Path,
 		intromw.InstrumentedHandler(OpenAPIV1Path, t.traceOpt, DiscoveryHandler()))
 	return nil
 }
 
-// configureDevMode configures the HttpTrasnport for
-// ComboMode.
+// ConfigureComboMode configures the HttpTransport for ComboMode.
 //
 // This mode runs both Indexer and Matcher in a single process.
 func (t *Server) configureComboMode(ctx context.Context) error {
@@ -157,7 +158,7 @@ func (t *Server) configureComboMode(ctx context.Context) error {
 	return nil
 }
 
-// configureIndexerMode configures the HttpTransport for IndexerMode.
+// ConfigureIndexerMode configures the HttpTransport for IndexerMode.
 //
 // This mode runs only an Indexer in a single process.
 func (t *Server) configureIndexerMode(_ context.Context) error {
@@ -166,11 +167,13 @@ func (t *Server) configureIndexerMode(_ context.Context) error {
 		return clairerror.ErrNotInitialized{Msg: "IndexerMode requires an indexer service"}
 	}
 
+	ratelimitMW := rate.NewRateLimitMiddleware(t.conf.Indexer.IndexReportRequestConcurrency)
+
 	t.Handle(AffectedManifestAPIPath,
 		intromw.InstrumentedHandler(AffectedManifestAPIPath, t.traceOpt, AffectedManifestHandler(t.indexer)))
 
 	t.Handle(IndexAPIPath,
-		intromw.InstrumentedHandler(IndexAPIPath, t.traceOpt, IndexHandler(t.indexer)))
+		intromw.InstrumentedHandler(IndexAPIPath, t.traceOpt, ratelimitMW.Handler(IndexAPIPath, IndexHandler(t.indexer))))
 
 	t.Handle(IndexReportAPIPath,
 		intromw.InstrumentedHandler(IndexReportAPIPath+"GET", t.traceOpt, IndexReportHandler(t.indexer)))
@@ -181,7 +184,9 @@ func (t *Server) configureIndexerMode(_ context.Context) error {
 	return nil
 }
 
-// configureMatcherMode configures HttpTransport
+// ConfigureMatcherMode configures HttpTransport for MatcherMode.
+//
+// This mode runs only a Matcher in a single process.
 func (t *Server) configureMatcherMode(_ context.Context) error {
 	// requires both an indexer and matcher service. indexer service
 	// is assumed to be a remote call over the network
@@ -189,8 +194,14 @@ func (t *Server) configureMatcherMode(_ context.Context) error {
 		return clairerror.ErrNotInitialized{Msg: "MatcherMode requires both indexer and matcher services"}
 	}
 
+	reportHandler := &VulnerabilityReportHandler{
+		Indexer: t.indexer,
+		Matcher: t.matcher,
+		Cache:   t.conf.Matcher.CacheAge,
+	}
+
 	t.Handle(VulnerabilityReportPath,
-		intromw.InstrumentedHandler(VulnerabilityReportPath, t.traceOpt, VulnerabilityReportHandler(t.matcher, t.indexer)))
+		intromw.InstrumentedHandler(VulnerabilityReportPath, t.traceOpt, reportHandler))
 
 	t.Handle(UpdateOperationAPIPath,
 		intromw.InstrumentedHandler(UpdateOperationAPIPath, t.traceOpt, UpdateOperationHandler(t.matcher)))
@@ -201,7 +212,9 @@ func (t *Server) configureMatcherMode(_ context.Context) error {
 	return nil
 }
 
-// configureMatcherMode configures HttpTransport
+// ConfigureNotifierMode configures HttpTransport for NotifierMode.
+//
+// This mode runs only a Notifier in a single process.
 func (t *Server) configureNotifierMode(ctx context.Context) error {
 	// requires both an indexer and matcher service. indexer service
 	// is assumed to be a remote call over the network
@@ -212,16 +225,11 @@ func (t *Server) configureNotifierMode(ctx context.Context) error {
 	t.Handle(NotificationAPIPath,
 		intromw.InstrumentedHandler(NotificationAPIPath, t.traceOpt, NotificationHandler(t.notifier)))
 
-	ks := t.notifier.KeyStore(ctx)
-	if ks == nil {
-		return clairerror.ErrNotInitialized{Msg: "NotifierMode requires the notifier to provide a non-nil key store"}
-	}
-
 	t.Handle(KeysAPIPath,
-		intromw.InstrumentedHandler(KeysAPIPath, t.traceOpt, KeysHandler(ks)))
+		intromw.InstrumentedHandler(KeysAPIPath, t.traceOpt, gone))
 
 	t.Handle(KeyByIDAPIPath,
-		intromw.InstrumentedHandler(KeyByIDAPIPath+"_KEY", t.traceOpt, KeyByIDHandler(ks)))
+		intromw.InstrumentedHandler(KeyByIDAPIPath+"_KEY", t.traceOpt, gone))
 
 	return nil
 }
@@ -230,10 +238,10 @@ func (t *Server) configureNotifierMode(ctx context.Context) error {
 // mint its own JWTs.
 const IntraserviceIssuer = `clair-intraservice`
 
-// configureWithAuth will take the current serve mux and wrap it
-// in an Auth middleware handler.
+// ConfigureWithAuth will take the current serve mux and wrap it in an Auth
+// middleware handler.
 //
-// must be ran after the config*Mode method of choice.
+// Must be ran after the config*Mode method of choice.
 func (t *Server) configureWithAuth(_ context.Context) error {
 	h, err := authHandler(&t.conf, t.Server.Handler)
 	if err != nil {
@@ -255,7 +263,7 @@ func unmodified(r *http.Request, v string) bool {
 	return false
 }
 
-// writerError is a helper that closes over an error that may be returned after
+// WriterError is a helper that closes over an error that may be returned after
 // writing a response body starts.
 //
 // The normal error flow can't be used, because the HTTP status code will have
@@ -277,3 +285,11 @@ func writerError(w http.ResponseWriter, e *error) func() {
 		w.Header().Add(errHeader, (*e).Error())
 	}
 }
+
+// SetCacheControl sets the "Cache-Control" header on the response.
+func setCacheControl(w http.ResponseWriter, age time.Duration) {
+	// The odd format string means "print float as wide as needed and to 0
+	// precision."
+	const f = `max-age=%.f`
+	w.Header().Set("cache-control", fmt.Sprintf(f, age.Seconds()))
+}
