@@ -2,32 +2,30 @@ package webhook
 
 import (
 	"context"
-	"fmt"
 	"net/http"
-	"time"
+	"sync"
 
 	"github.com/google/uuid"
 	"github.com/quay/zlog"
 	"go.opentelemetry.io/otel/baggage"
 	"go.opentelemetry.io/otel/label"
-	"gopkg.in/square/go-jose.v2"
-	"gopkg.in/square/go-jose.v2/jwt"
 
 	clairerror "github.com/quay/clair/v4/clair-error"
 	"github.com/quay/clair/v4/internal/codec"
 	"github.com/quay/clair/v4/notifier"
-	"github.com/quay/clair/v4/notifier/keymanager"
 )
 
+// SignedOnce is used to print a deprecation notice, but only once per run.
+var signedOnce sync.Once
+
 type Deliverer struct {
 	conf Config
 	// a client to use for POSTing webhooks
-	c    *http.Client
-	kmgr *keymanager.Manager
+	c *http.Client
 }
 
 // New returns a new webhook Deliverer
-func New(conf Config, client *http.Client, keymanager *keymanager.Manager) (*Deliverer, error) {
+func New(conf Config, client *http.Client) (*Deliverer, error) {
 	var c Config
 	var err error
 	if c, err = conf.Validate(); err != nil {
@@ -39,7 +37,6 @@ func New(conf Config, client *http.Client, keymanager *keymanager.Manager) (*Del
 	return &Deliverer{
 		conf: c,
 		c:    client,
-		kmgr: keymanager,
 	}, nil
 }
 
@@ -47,33 +44,6 @@ func (d *Deliverer) Name() string {
 	return "webhook"
 }
 
-// sign will use the provided private key to sign and attach a jwt to the provided
-// request.
-func (d *Deliverer) sign(ctx context.Context, req *http.Request, kp keymanager.KeyPair) error {
-	opts := (&jose.SignerOptions{}).
-		WithType("JWT").
-		WithHeader(jose.HeaderKey("kid"), kp.ID.String())
-	signer, err := jose.NewSigner(jose.SigningKey{Algorithm: jose.PS512, Key: kp.Private}, opts)
-	if err != nil {
-		return fmt.Errorf("failed to create jwt signer: %v", err)
-	}
-	now := jwt.NumericDate(time.Now().Unix())
-	expire := jwt.NumericDate(time.Now().Add(1 * time.Hour).Unix())
-	cl := jwt.Claims{
-		Issuer:   "notifier",
-		Expiry:   &expire,
-		IssuedAt: &now,
-		Audience: jwt.Audience{d.conf.target.String()},
-		Subject:  d.conf.target.Hostname(),
-	}
-	token, err := jwt.Signed(signer).Claims(cl).CompactSerialize()
-	if err != nil {
-		return err
-	}
-	req.Header.Set("Authorization", "Bearer "+token)
-	return nil
-}
-
 // Deliver implements the notifier.Deliverer interface.
 //
 // Deliver POSTS a webhook data structure to the configured target.
@@ -102,15 +72,11 @@ func (d *Deliverer) Deliver(ctx context.Context, nID uuid.UUID) error {
 
 	// sign a jwt using key manager's private key
 	if d.conf.Signed {
-		kp, err := d.kmgr.KeyPair()
-		if err != nil {
-			return fmt.Errorf("configured for signing but no private key available: %v", err)
-		}
-		err = d.sign(ctx, req, kp)
-		if err != nil {
-			return fmt.Errorf("failed to sign request: %v", err)
-		}
-		zlog.Debug(ctx).Msg("successfully signed request")
+		signedOnce.Do(func() {
+			zlog.Warn(ctx).Msg(`"signed" configuration key no longer does anything`)
+			zlog.Warn(ctx).Msg(`specifying "signed" will be an error in the future`)
+			// Make good on this threat in... 4.4?
+		})
 	}
 
 	zlog.Info(ctx).
